# (C) 2001-2017 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.



#====================================================================================
# This script is to test various modes of S10 KR PHY in hardware 
# This script should be run in the System Console window
# Type the command "source $PATH/console.tcl" in the System Console window to execute
#====================================================================================
global jtag_master

#==============================================================================
# Some global variables here 
#==============================================================================
# Change declaration of ch_all/th_all below if no of channels/test-harness are changed in design example
set ch_all [list 0 1]
set th_all [list 0]

#==============================================================================
# reconfig_read           
# This proc needs address and slave-name is input arguments
# Slave name is determined by proc "set_up_channels"
#==============================================================================
proc reconfig_read { channel address_hex } {
    global jtag_master
    set address_hex [expr $address_hex | [expr $channel<<12]]	
    set data [master_read_32 $jtag_master [expr {$address_hex*4}] 1] 
    return $data
}
#==============================================================================
# reconfig_write          
# This proc needs address, data, and slave-name is input arguments
# Slave name is determined by proc "set_up_channels"
#==============================================================================
proc reconfig_write { channel address_hex value} {
    global jtag_master
    set address_hex [expr $address_hex | [expr $channel<<12]]	
    master_write_32 $jtag_master [expr {$address_hex*4}] $value 
}

# help proc read-modify-write RMW
proc rdmwr {channel address  bitmask newval} {
    set address [expr $address | [expr $channel<<12]]	
    set value [reconfig_read $channel $address]
    set value [expr $value & [expr 0xFFFFFFFF & ~$bitmask]]
    set value [expr $value | $newval]

    reconfig_write $channel $address $value
}
#______________________________________________________________________________
# reconfig access for dashboard
#______________________________________________________________________________
proc reg_read {base {offset 0}} {
    global jtag_master

    set word_address [ expr $base + $offset ]
    set byte_address [ expr $word_address * 4 ]
    set rdata [master_read_32 $jtag_master $byte_address 1]
    return $rdata
}
proc reg_write {base args} {
    global jtag_master

    set arg_list [regexp -all -inline {\S+} $args]
    set num_args [llength $arg_list]

    if {$num_args == 1} {
        set offset 0
        set wdata [lindex $arg_list 0]
    } elseif {$num_args == 2} {
        set offset [lindex $arg_list 0]
        set wdata  [lindex $arg_list 1]
    } else {
        puts "reg_write: rong number of arguments"
        return -1
    }

    set word_address [ expr $base + $offset ]
    set byte_address [ expr $word_address * 4 ]
    master_write_32 $jtag_master $byte_address $wdata
    return 0
}
proc reconfig_reg_read {chan addr offset} {
       
    set chanoffs [expr $offset + (0x800 * $chan)]
    return [reg_read $chanoffs $addr]
}

proc reconfig_reg_write {chan addr offset val} {
        
    set chanoffs [expr $offset + (0x800 * $chan)]
    return [reg_write $chanoffs $addr $val]
}

proc reconfig_reg_rmw {chan addr offset mask val} {
    set value [reconfig_reg_read $chan $addr $offset]
    set value [expr $value & [expr 0xffffffff & ~$mask]] 
    set value [expr $value | $val] 
    reconfig_reg_write $chan $addr $offset $value
}
#______________________________________________________________________________
#================================================================================
#  create dummy adme_read/write proc- so prbs_testing_suite doesnt have to change
#================================================================================
proc adme_write { channel address_hex value} {
    reconfig_write $channel $address_hex $value	
}

proc adme_read { channel address_hex } {
    reconfig_read $channel $address_hex
}
#==============================================================================
#  Re-start the link without clearing the registers
#  need read-modify-write as reset bit is shared
#==============================================================================
proc rst {channel} {
global ch_all
    set rd_data    [reconfig_read [lindex $ch_all $channel] 0xcb0];
    set wr_data    [expr $rd_data | 1];
    set data_hex   [format "%#x" $wr_data];
    reconfig_write [lindex $ch_all $channel] 0x0cb0 $data_hex;        # seq_reset for channel
}


proc rst_all {} {
global ch_all
set i 0
    foreach ad $ch_all {
        rst $i
	incr i
    }
}

#============= Enable/Disable serial loopback =================================
proc loop_on {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xc61 1
    }
}

proc loop_off {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xc61 0
    }
}	
#==============================================================================
#  Disable the Sequencer AN timeout, may get stuck if AN has errors
#==============================================================================
proc unh_dis_seqan_timer {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xcb0 2
    }
}

proc unh_en_seqan_timer {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xcb0 0
    }
}
#==============================================================================
#  Disable the LT PMA initialize on max_wait_timeout (bit 15 in 0xD0), 
#==============================================================================
proc dis_max_wait_timer {} {
global ch_all
    foreach ad $ch_all {
        rdmwr $ad 0xcd0 0x2 0x2
    }
}	
#==============================================================================
# Force the data mode for the specific channel
# channel = 0,1,2;  mode = 0 for no force, 1 for Gige, 4 for 10G
#==============================================================================
proc seq_force {channel mode} {
global ch_all
    set address     0xcb0
    set value       [expr 0x106 |[expr [expr $mode << 4] + 1]];      # also set seq reset,recal
    set value_hex   [format "%#x" $value];
    reconfig_write  [lindex $ch_all $channel] $address $value_hex;
}

#==============================================================================
#  Disable AN on all channels
#==============================================================================
proc dis_an {} {
global ch_all
    foreach ad $ch_all {
        reconfig_write $ad 0xcc0 0
    }
}

proc dis_nonce {} {
global ch_all
    foreach ad $ch_all {
	rdmwr $ad 0xcc0 0x80 0x80 
    }
}
#==============================================================================
#  Read 10G status
#==============================================================================
proc rd_10g_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "address 0x482 for ch$i == [reconfig_read $ad 0xc82]";
        incr i
    }
}

#==============================================================================
#  FEC Section here -- inject and clear FEC errors
#==============================================================================

#  Clear 10G HSSI error counters 10G/10G-FEC for all channels
proc clr_hssi_err_all { } {
global ch_all
set i 0
    foreach ad $ch_all {
        clr_hssi_err $ad 
        incr i
    }
}

#  Clear 10G HSSI error counters 10G/10G-FEC
proc clr_hssi_err { channel } {
   puts "Clearing error counters"
   set address     0xC81
   reconfig_write $channel $address 0x4 
   reconfig_write $channel $address 0x0 
}

#  Read FEC error counts
proc rd_fec_errors { channel} {
   set add_corr     0xDC
   set corr_err     [reconfig_read $channel $add_corr]
   set add_uncr     0xE0
   set uncr_err     [reconfig_read $channel $add_uncr]
   puts "FEC errors for channel $channel:- Correctable:- $corr_err   Uncorrectable:- $uncr_err"
}

# Read FEC error counts for all channels
proc rd_fec_errors_all {} {
global ch_all
set i 0
    foreach ad $ch_all {
        rd_fec_errors $i
        incr i
    }
}


#  Inject 1 FEC errors for all burst lenghts for all channels
proc inj_traffic_err_all { } {
global ch_all
set i 0
    foreach ad $ch_all {
        inj_traffic_err $i
        incr i
    }
}

#  Inject 1 FEC errors for all burst lenghts
proc inj_traffic_err { channel } {
set x 0
   puts "Inject 1 error- across all burst lenghts for channel $channel"
   set add_bl       0xBD
   set add_inj      0xCB2
   for {set x 0} {$x < 16} {incr x} {
      reconfig_write $channel $add_bl  [expr 0x0E & $x*16]
      reconfig_write $channel $add_inj 0x800
      # Wait for 1 ms before inserting another error
      # This will make sure errors are inserted in different FEC frames
      after 1
   }
}



#==============================================================================
#  Read status for all channels
#==============================================================================
# Sequencer status registers 0xB0 and 0xB1
proc rd_seq_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "Channel:$i  Address 0x4B0 == [reconfig_read $ad 0xcb0]";
        puts "           Address 0x4B1 == [reconfig_read $ad 0xcb1]";
        incr i
    }
}

# Auto-Neg status registers 0xC0 and 0xC2
proc rd_an_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "Channel:$i Address 0x4C0 == [reconfig_read $ad 0xCc0]";
        puts "          Address 0x4C2 == [reconfig_read $ad 0xCc2]";
        incr i
    }
}

# link training status registers 0xD0 and 0xD2
proc rd_lt_stat {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "Channel:$i Address 0xcD0 == [reconfig_read $ad 0xcd0]";
        puts "          Address 0xcD2 == [reconfig_read $ad 0xcd2]";
        incr i
    }
}

# link training tap values
# Read PRE/POST/VOD values for all channel from KR IP CSRs
proc rd_lt_taps_ip {} {
global ch_all
set channel 0
    foreach ad $ch_all {
        set rd_data  [reconfig_read $ad 0xcd5];
        set vod      [expr $rd_data & 0x1F]
        set post     [expr $rd_data & 0x3F00]
        set post     [expr $post >> 8]
        set pre      [expr $rd_data & 0XF0000]
        set pre      [expr $pre >> 16]
        set ctle     [expr $rd_data & 0XF00000]
        set ctle     [expr $pre >> 20]
        set ctle_md  [expr $rd_data & 0X3000000]
        set ctle_md  [expr $pre >> 24]
        set dfe      [expr $rd_data & 0XC000000]
        set dfe      [expr $pre >> 26]
        puts "KRIP CSRs for Channel-$channel === VOD:-$vod  POST:-$post  PRE:-$pre"
        incr channel
    }
}

# Read PRE/POST/VOD values for all channel from HSSI DPRIO
proc rd_lt_taps_hssi_all {} {
global ch_all
    foreach ad $ch_all {
    rd_lt_taps_hssi $ad 
    }
}

# Read PRE/POST/VOD values for given channel from HSSI DPRIO
# Post process and print
proc rd_lt_taps_hssi {channel} {
   set post [rd_post_tap $channel] 
   set pre  [rd_pre_tap  $channel] 
   set vod  [rd_vod_tap  $channel] 
   puts "HSSI DPRIO for Channel-$channel === VOD:-$vod  POST:-$post  PRE:-$pre"
}

proc rd_post_tap {slave} {
   set add       0x105
   set rd_data   [reconfig_read $slave $add];
   set rd_data  [expr $rd_data & 0x3F]
   return $rd_data
}   

proc rd_pre_tap {slave} {
   set add      0x107
   set rd_data  [reconfig_read $slave $add];
   set rd_data  [expr $rd_data & 0x1F]
   return $rd_data
}

proc rd_vod_tap {slave} {
   set add       0x109
   set rd_data    [reconfig_read $slave $add];
   set rd_data  [expr $rd_data & 0x1F]
   return $rd_data
}

#==============================================================================
# KRIP related proc here 
#==============================================================================
# link training error accumulation time
proc rd_lt_bertime {} {
global ch_all
set i 0
    foreach ad $ch_all {
        puts "address 0x4D3 for channel$i== [reconfig_read $ad 0xcD3]";
        incr i
    }
}

proc rd_all_stat {} {
  rd_seq_stat
  rd_an_stat
  rd_lt_stat
  rd_lt_bertime
  rd_lt_taps_ip
  rd_lt_taps_hssi_all
}

#==============================================================================
#  Test harness proceduers - start test/read status etc
#==============================================================================
# start 10G packets test for th0
proc start_xgmii_th0 {} {
global th_all
  reconfig_write 0 0xF000 0x01
}

## start 10G packets test for th2
#proc start_xgmii_th2 {} {
#global th_all
#  reconfig_write 0 0xE000 0x01
#}

# Read non idle data sent during the test- expected value is 418
proc rd_pkt_cnt_th0 {} {
global th_all
  set cnt [reconfig_read 0 0xF004]
  puts "Non-idle count for Channel-0 is :- $cnt"  
  set cnt [reconfig_read 0 0xF005]
  puts "Non-idle count for Channel-1 is :- $cnt"  
}

## Read non idle data sent during the test- expected value is 418
#proc rd_pkt_cnt_th2 {} {
#global th_all
#  set cnt [reconfig_read 0 0xE004]
#  puts "Non-idle count for Channel-2 is :- $cnt"  
#  set cnt [reconfig_read 0 0xE005]
#  puts "Non-idle count for Channel-3 is :- $cnt" 
#}

#==============================================================================
#  Help procedures here
#==============================================================================
proc de_script_intro {} {
  puts "-----------------------------------------------------------------------------------------"
  puts "This is a PCS-only PHY IP design for testing Stratix-10 10G-KR PHY"
  puts "Auto-Negotiation (Clause 73),Link Training (Clause 72) and FEC (Clause 74) are supported"
  puts "-----------------------------------------------------------------------------------------"
}

proc de_script_help {} {
  puts " "
  puts "Channel 0 == address 0x0Cnn == GXBR_4C_RX0/TX0 SMAs on the Si Dev kit"
  puts "Channel 1 == address 0x1Cnn == GXBR_4C_RX1/TX1 SMAs on the Si Dev kit"
  puts " "
  puts "Test Harness 0 == address 0xFnnn"
  puts " "
  puts "Use the \" reconfig_write 0 address  data\" command to write a register in Channel-0"
  puts "Use the \" reconfig_read  0 address\"       command to read a register in Channel-0 - address 0x4nn"
  puts "Use the \" reconfig_read  1 address\"       command to read a register in Channel1  - address 0x4nn"
  puts "Use the \" reconfig_write 0 address  data\" command to write a register in Test-hardness-0"
  puts "Use the \" reconfig_read  0 address\"       command to read a register in Test-hardness-0 - address 0xFnnn"
  puts "The address and data need to be hexadecimal i.e. 0xnnn where n=0-f"
  puts " "
  puts "Use the \" rst_all\"   command to Re-start the link"
  puts " "
}



#==============================================================================
#                       Open and close JTAG
#==============================================================================
proc open_jtag { index } {
    global jtag_master
    global jtag_master_list
    set jtag_master_list [get_service_paths master] ;
    # Change following expression as needed
    set jtag_master [lindex $jtag_master_list $index]
    open_service master $jtag_master;
    puts "Selected JTAG MASTER is $jtag_master"
    open_service master $jtag_master;
    puts "\nInfo: Opened JTAG Master Service\n"
}

proc get_jtag_masters {} {
    global jtag_master
    global jtag_master_list
    set jtag_master_list [get_service_paths master] ;
    set i 0
    foreach m $jtag_master_list  {
      puts "${i}) $m"	     
      incr i
    }	    
}	
proc close_jtag {} {
    global jtag_master
    close_service master $jtag_master;
    puts "\nInfo: Closed JTAG Master Service\n\n";
}

#==============================================================================
#==============================================================================
#  Main program starts here 
#==============================================================================
#==============================================================================

  #design_load de_wrapper.sof
  de_script_intro
  de_script_help
  puts "Use the \"de_script_help\"   command to see the above message"
  puts " "
  get_jtag_masters
  puts "Use command \" get_jtag_masters \" to get list of available Jtag Masters"
  puts "Use command \" open_jtag <number> \" to open Jtag Service"
  puts "e.g. \" open_jtag 0 \" will open first Jtag returned by command \"get_jtag_masters \". "
  #open_jtag
  #rst_all



#==============================================================================
